#include "AnimacaoPose.hpp"

// Atualiza uma unica pose da face
void AnimacaoPose::atualizaPose(int pose, double inf)
{
   face->atualizaAnimacaoManual(pose, inf);
}

// Executa o processo de interpolacao do estado atual com a pose passada no
// parametro, de acordo com sua respectiva influencia, numa velocidade dada
void AnimacaoPose::interpola(int pose, double infFinal, double vel)
{
   if (!encerrouAnimacao())
   {
      // Por convencao, um valor negativo da pose dispara a animacao de
      // desfazer esta pose
      if (pose < 0)
      {
         interpolaInverso(-pose, infFinal, vel);
         return;
      }

      // Incrementa o contador da influencia atual ate que ele seja igual ao
      // valor desejado, exibindo as poses com estas influencias intermediarias
      if (infAtual <= infFinal)
      {
         atualizaPose(pose, infAtual);
         infAtual += vel;
      }
      // Encerrou a animacao
      else
      {
         // Guarda a influencia da pose passada
         atualizaPose(pose, infFinal);
         face->armazenaInfluenciaPose(pose, infFinal);
         encerraAnimacao();
      }
   }
}

// Desfaz uma pose, isso e, decrementa sua influencia, ate a face neutra
void AnimacaoPose::interpolaInverso(int pose, double infFinal, double vel)
{
   if (!encerrouAnimacao())
   {
      // Neste caso, a influencia passada sera a influencia armazenada, menos
      // o contador
      if (infAtual >= 0)
      {
         atualizaPose(pose, infAtual);
         infAtual -= vel;
      }
      // Encerrou a animacao
      else
      {
         // A pose foi desfeita, zera sua influencia no vetor de influencias das
         // poses
         atualizaPose(pose, 0.0);
         face->armazenaInfluenciaPose(pose, 0.0);
         encerraAnimacao();
      }
   }
}

// Exibe a pose passada por parametro e retorna para a pose neutra
void AnimacaoPose::interpolaDesfaz(int pose, double infFinal, double vel)
{
   if (!encerrouAnimacao())
   {
      // Ida: interpola a pose
      if (!desfaz)
         interpola(pose, infFinal, vel);
      // Volta: interpola a pose complementar
      else
         interpolaInverso(pose, infFinal, vel);
      if (encerrouAnimacao())
      {
         desfaz = !desfaz;
         // Se a AnimacaoPose da ida acabou, inicia a AnimacaoPose da volta
         if (desfaz)
            iniciaAnimacao(face->devolveInfluenciaPose(pose));
      }
   }
}

// Algumas acoes do avatar devem acontecer repetidamente em intervalos de tempo
// aleatorios (como piscar, por exemplo). Sorteia o novo tempo para o inicio da
// proxima animacao
void AnimacaoPose::sorteiaNovoTempo(int cntFrames, int tMin, int tDelta)
{
   if (encerrouAnimacao())
   {
      // O contador de frames e usado para definir a hora de animar
      if (cntFrames % tempo == 0)
      {
         iniciaAnimacao(0);

         // Sorteia o novo tempo aleatorio para iniciar a proxima animacao,
         // que acontecera no intervalo de [tMin, tMin + tDelta] frames
         srand(time(NULL));
         tempo = rand() % tDelta + tMin;
      }
   }
}

