#ifndef __APLICACAOBASE_H__
#define __APLICACAOBASE_H__

#include "Ogre.h"
#include "OgreConfigFile.h"
#include "FrameListenerBase.hpp"

#define TELA_X 600
#define TELA_Y 450
#define TELA_RES_X 1280
#define TELA_RES_Y 800

using namespace Ogre;

// Classe que configura a base para a inicializacao de um programa do Ogre
// Baseada na implementacao do arquivo ExampleApplication.h da engine 
class AplicacaoBase
{
public:
   AplicacaoBase();
   ~AplicacaoBase();

   // Roda o programa
   void inicia();

   // Metodos de configuracao
   bool configura();
   bool configuraJanela();
   void criaViewports();
   void configuraRecursos();
   void carregaRecursos();
   
   // Deixa as tarefas especificas da aplicacao a cargo de classes herdeiras
   // (configuracao da cena e fluxo do programa)
   virtual void criaLuz() {};
   virtual void criaCamera() {}
   virtual void criaCena() {};
   virtual void criaFrameListener() {};
  
protected:
   // O objeto root e o ponto de partida para o desenvolvimento de uma aplicacao
   // do Ogre
   Root* root;

   // Objeto responsavel por gerenciar a renderizacao de uma cena
   SceneManager* sceneMgr;

   // Receptor de eventos
   FrameListenerBase* frameListener;

   // Janela onde roda a aplicacao
   RenderWindow* janela;

   // Camera da cena
   Camera* camera;
   
   // Caminho para os arquivos de configuracao
   String caminho;

   // Variavel para verificar se a camera sera usada
   bool usaCamera;
};


#endif
