#include "Audio.hpp"

// O objeto audio e um singleton, criado apenas uma vez
template<> Audio* Singleton<Audio>::ms_Singleton = 0;

// Construtor
Audio::Audio()
{
   fala = false;
   fonema = "";
}

// Calback chamada automaticamente pelo eSpeak quando um buffer de audio e
// produzido
int callbackDeSintese(short* wav, int numEntradas, espeak_EVENT* eventos)
{
   // Percorre a lista de eventos obtida na sintese
   for(;eventos->type != 0; eventos++)
   {
      // Verifica se recebeu um evento de fonema
      if (eventos->type == espeakEVENT_PHONEME)
         Audio::getSingleton().idParaString(eventos->id.number);
   }

   // Continua a sintese
   return 0;
}

// Inicializacao do Audio
void Audio::inicializaAudio()
{
   // Inicializa
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
    espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS, 0, NULL, 1);
#else
    espeak_Initialize(AUDIO_OUTPUT_SYNCH_PLAYBACK, 0, NULL, 1);
#endif

    // Define a callback de sintese
    espeak_SetSynthCallback(callbackDeSintese);

    // Tenta pronunciar alguma coisa para verificar se a inicializacao foi
    // feita com sucesso
    try {
       pronuncia("");

    } catch(Exception& e)
    {
       printf("Erro na inicializacao do audio.\nVerifique se nao existe outro programa aberto que esteja utilizando som.\n");
    }
}

// Define a voz para a face
void Audio::defineVoz(Face* face)
{
   // Voz da Ana: em portugues com variante +f3 (feminina)
   if (face->getNome() == "Ana")
   {
      espeak_SetVoiceByName("pt+f3");
      // Ajusta a velocidade
      espeak_SetParameter(espeakRATE, 90, 0);
   }

   // Voz do Caio: em portugues com variante +m2 (masculina)
   else if (face->getNome() == "Caio")
   {
      espeak_SetVoiceByName("pt+m2");
      espeak_SetParameter(espeakRATE, 100, 0);
   }

   // Voz da Helena: em portugues com variante +f4 (feminina)
   else if (face->getNome() == "Helena")
   {
      espeak_SetVoiceByName("pt+f4");
      espeak_SetParameter(espeakRATE, 90, 0);
   }
   // Voz padrao (masculina)
   else
   {
      espeak_SetVoiceByName("pt");
      espeak_SetParameter(espeakRATE, 100, 0);
   }
}

// Sintetiza a pronuncia um texto
void Audio::pronuncia(const char* texto)
{
   espeak_Synth(texto, sizeof(texto) + 1, 0, POS_CHARACTER, 0, espeakPHONEMES, 
         NULL, NULL);
}

// No evento do tipo espeakEVENT_PHONEME, o fonema possui um id.number que o
// identifica. Converte esse id para a string correspondente ao fonema
void Audio::idParaString(unsigned int palavra)
{
   static char buf[5];

   for (int i = 0; i < 3; i++)
      buf[i] = palavra >> (i*8);
   buf[4] = 0;

   // Armazena o fonema da vez
   fonema = buf;
}

// Retorna o fonema que a face esta pronunciando no momento
const char* Audio::fonemaAtual()
{
   return fonema;
}

// Metodos para controle da fala
void Audio::iniciaFala()
{
   fala = true;
}

void Audio::encerraFala()
{
   fala = false;
}

bool Audio::estaFalando()
{
   return fala;
}


