#include "Avator.hpp"

// Construtor
Avator::Avator(bool camera)
{
   // Inicializa o audio
   new Audio;

   // Inicializa a camera, se houver a opcao de camera
   usaCamera = camera;
   if (usaCamera == true)
      new Visao;
}

// Destrutor
Avator::~Avator()
{
   // Destroi o sistema GUI
   if (mSistemaGUI)
   {
      delete mSistemaGUI;
      mSistemaGUI = 0;
   }
   // Destroi o renderizador GUI
   if (mRenderizadorGUI)
   {
      delete mRenderizadorGUI;
      mRenderizadorGUI = 0;
   }

   if (usaCamera)
      Visao::getSingleton().~Visao();
}

// Cria e configura a camera
void Avator::criaCamera()
{
   // Cria a camera
   camera = sceneMgr->createCamera("CameraPrincipal");

   // Define a posicao da camera, ao longo do eixo Z
   camera->setPosition(0, 0, 400);
   // Define a posicao para onde a camera aponta, no centro da tela
   camera->lookAt(0, 0, 0);
}

// Cria a luz ambiente
void Avator::criaLuz()
{
   // Cria luz de cor branca
   luz = sceneMgr->createLight("LuzAmbiente");
   sceneMgr->setAmbientLight(ColourValue(1.0, 1.0, 1.0));

   // Define as propriedades da luz
   luz->setPosition(Vector3(20, 50, 250));
   luz->setDiffuseColour(ColourValue::White);
   luz->setSpecularColour(ColourValue::White);
}

// Configuracoes iniciais do sistema GUI
void Avator::inicializaSistemaGUI()
{
   // Inicializacao
   mRenderizadorGUI = new CEGUI::OgreCEGUIRenderer(janela, 
         Ogre::RENDER_QUEUE_OVERLAY, false, 0, sceneMgr);
   mSistemaGUI = new CEGUI::System(mRenderizadorGUI);
   
   // Configura o log de mensagens do CEGUI
   CEGUI::Logger::getSingleton().setLoggingLevel(CEGUI::Informative);

   // Carrega o esquema Taharez (padrao do CEGUI)
   CEGUI::SchemeManager::getSingleton().loadScheme(
         (CEGUI::utf8*)"TaharezLookSkin.scheme");

   // Define o cursor do mouse
   mSistemaGUI->setDefaultMouseCursor((CEGUI::utf8*)"TaharezLook", 
         (CEGUI::utf8*)"MouseArrow");

   // Define a fonte usada
   mSistemaGUI->setDefaultFont((CEGUI::utf8*)"Verdana");
   
   // Carrega o layout do programa
   CEGUI::WindowManager& wmgr = CEGUI::WindowManager::getSingleton();
   CEGUI::Window* sheet = wmgr.loadWindowLayout((CEGUI::utf8*)"Avator.layout");
   mSistemaGUI->setGUISheet(sheet);
}

// Insere uma caixa de texto para que o usuario possa interagir com o sistema
void Avator::inicializaCaixaDeTexto()
{
   // Cria a caixa, do tipo Editbox do CEGUI
   CEGUI::WindowManager& wmgr = CEGUI::WindowManager::getSingleton();
   caixaDeTexto = static_cast<CEGUI::Editbox*>(wmgr.getWindow("Editbox"));
   
   // Define o tamanho maximo do texto da caixa
   caixaDeTexto->setMaxTextLength(MAXCHAR);

   // Define que o conteudo da caixa pode ser alterado pelo usuario 
   caixaDeTexto->setReadOnly(false);

   // Desabilita a propriedade de mascara (que esconderia o texto no caso de
   // uma senha, por exemplo)
   caixaDeTexto->setTextMasked(false);
}

// Cria uma lista contendo os avatares disponiveis
void Avator::inicializaListaAvatares()
{
   // Cria a lista como sendo um widget do tipo combobox
   CEGUI::Combobox* lista = (CEGUI::Combobox*)
      CEGUI::WindowManager::getSingleton().getWindow("Nomes");

   // Define a callback de tratamento da combobox
   lista->subscribeEvent(CEGUI::Combobox::EventListSelectionAccepted, 
            CEGUI::Event::Subscriber(&Avator::trataMudancaAvatar, this));

   // Adiciona os avatares existentes
   CEGUI::ListboxTextItem* item;
   item = new CEGUI::ListboxTextItem((CEGUI::utf8*)"Ana", 0);
   lista->addItem(item);
   item = new CEGUI::ListboxTextItem((CEGUI::utf8*)"Caio", 1);
   lista->addItem(item);
   item = new CEGUI::ListboxTextItem((CEGUI::utf8*)"Helena", 2);
   lista->addItem(item);
}

// Define o tratamento de eventos de selecao na combobox
bool Avator::trataMudancaAvatar(const CEGUI::EventArgs& e)
{
   // Nao muda de modelo se o avatar atual estiver falando
   if (!Audio::getSingleton().estaFalando())
   {
      // Seleciona o nome
      CEGUI::ListboxItem* item = ((CEGUI::Combobox*)
            ((const CEGUI::WindowEventArgs&)e).window)->getSelectedItem();

      // Converte de CEGUI::String para Ogre::String
      Ogre::String meshName(item->getText().c_str());

      // Remove o avatar atual
      face->removeAnimacoesFace();
      face->removeFaceCena(sceneMgr);

      // Insere o avatar selecionado
      face->setNome(meshName);
      try
      {
         face->recebeComponentesFace(sceneMgr);
      }
      // Se os componentes ainda nao foram carregados, carrega
      catch (Ogre::Exception e)
      {
         face->carregaComponentesFace();
      }
      face->criaAnimacoesFace();
      face->criaEntidadesFace(sceneMgr);
      face->inicializaAnimacoesFace();
      face->anexaFaceCena();

      // Atualiza a voz
      Audio::getSingleton().defineVoz(face);
   }
   return true;
}

// Cria a cena, dispondo a face e a caixa de texto
void Avator::criaCena()
{
   // Cria a face. Por padrao a face da Ana e a primeira a ser exibida
   face = new Face();
   face->setNome("Ana");

   // Insere a face
   face->carregaComponentesFace();
   face->criaAnimacoesFace();
   face->criaEntidadesFace(sceneMgr);
   face->inicializaAnimacoesFace();
   face->criaNoFace(sceneMgr, Vector3(0, 0, 0));
   face->anexaFaceCena();

   // Ajuste inicial da rotacao da face, para que ela fique de frente para a
   // camera
   face->rotaciona('X', 110);

   // Inicializa o audio
   Audio::getSingleton().inicializaAudio();
   Audio::getSingleton().defineVoz(face);

   // Inicializa a camera
   if (usaCamera)
      Visao::getSingleton().inicializaVisao();

   // Insere a caixa de texto e a lista de avatares
   inicializaSistemaGUI();
   inicializaCaixaDeTexto();
   inicializaListaAvatares();
}

// Adiciona um frame listener
void Avator::criaFrameListener()
{
   // Cria o listener
   frameListener = new FrameListenerAvator(janela,face,caixaDeTexto,usaCamera);
   
   // Registra o listener
   root->addFrameListener(frameListener);
}

