#include "Face.hpp"

// Construtor
Face::Face()
{
   // Inicialmente, a face esta na pose neutra
   for (int i = 0; i < NUM_POSES; i++)
      infPose[i] = 0;
}

// Mapeia as acoes enumeradas para os comandos correspondentes, chamando-os com
// uma dada influencia
void Face::atualizaAnimacaoManual(int acao, double influencia)
{
   // Expressoes
   if (acao == RAIVA)
      exibeExpressaoRaiva(influencia);

   else if (acao == NOJO)
      exibeExpressaoNojo(influencia);

   else if (acao == MEDO)
      exibeExpressaoMedo(influencia);

   else if (acao == TRISTEZA)
      exibeExpressaoTristeza(influencia);

   else if (acao == SORRISO_FECHADO)
      exibeExpressaoSorrisoFechado(influencia);

   else if (acao == SORRISO_ABERTO)
      exibeExpressaoSorrisoAberto(influencia);

   else if (acao == SURPRESA)
      exibeExpressaoSurpresa(influencia);

   // Visemas
   else if (acao == VISEMA_A)
      exibeVisemaA(influencia);

   else if (acao == VISEMA_BMP)
      exibeVisemaBMP(influencia);

   else if (acao == VISEMA_DST)
      exibeVisemaDST(influencia);

   else if (acao == VISEMA_E)
      exibeVisemaE(influencia);

   else if (acao == VISEMA_FV)
      exibeVisemaFV(influencia);

   else if (acao == VISEMA_I)
      exibeVisemaI(influencia);

   else if (acao == VISEMA_JX)
      exibeVisemaJX(influencia);

   else if (acao == VISEMA_KG)
      exibeVisemaKG(influencia);

   else if (acao == VISEMA_LN)
      exibeVisemaLN(influencia);

   else if (acao == VISEMA_O)
      exibeVisemaO(influencia);

   else if (acao == VISEMA_R)
      exibeVisemaR(influencia);

   else if (acao == VISEMA_U)
      exibeVisemaU(influencia);

   // Acoes
   else if (acao == PISCAR_ESQ)
      exibeAcaoPiscarEsq(influencia);

   else if (acao == PISCAR_DIR)
      exibeAcaoPiscarDir(influencia);

   else if (acao == ABAIXAR_SOBR_ESQ)
      exibeAcaoAbaixarSobrancelhaEsq(influencia);

   else if (acao == ABAIXAR_SOBR_DIR)
      exibeAcaoAbaixarSobrancelhaDir(influencia);

   else if (acao == LEVANTAR_SOBR_ESQ)
      exibeAcaoLevantarSobrancelhaEsq(influencia);

   else if (acao == LEVANTAR_SOBR_DIR)
      exibeAcaoLevantarSobrancelhaDir(influencia);

   else if (acao == APERTAR_OLHO_ESQ)
      exibeAcaoApertarOlhoEsq(influencia);

   else if (acao == APERTAR_OLHO_DIR)
      exibeAcaoApertarOlhoDir(influencia);

   // Movimento dos olhos
   else if (acao == OLHAR_BAIXO)
      exibeMovimentoOlharBaixo(influencia);

   else if (acao == OLHAR_CIMA)
      exibeMovimentoOlharCima(influencia);

   else if (acao == OLHAR_ESQ)
      exibeMovimentoOlharEsq(influencia);

   else if (acao == OLHAR_DIR)
      exibeMovimentoOlharDir(influencia);

   // Acoes compostas
   else if (acao == PISCAR)
      exibeAcaoPiscar(influencia);

   else if (acao == ABAIXAR_SOBRANCELHA)
      exibeAcaoAbaixarSobrancelha(influencia);

   else if (acao == LEVANTAR_SOBRANCELHA)
      exibeAcaoLevantarSobrancelha(influencia);

   else if (acao == APERTAR_OLHO)
      exibeAcaoApertarOlho(influencia);
}

// Na animacao automatica da face, apenas atualiza a animacao de todos os seus
// componentes
void Face::atualizaAnimacaoAutomatica(const FrameEvent &evt)
{
   olhoEsquerdo->atualizaPoseAutomatica(evt);
   olhoDireito->atualizaPoseAutomatica(evt);
   pele->atualizaPoseAutomatica(evt);
   denteBaixo->atualizaPoseAutomatica(evt);
   denteCima->atualizaPoseAutomatica(evt);
   lingua->atualizaPoseAutomatica(evt);
   interiorBoca->atualizaPoseAutomatica(evt);
}

// Comandos de animacao da face:
// Os comandos variam com relacao aos componentes da face em que atuam. Por
// exemplo, alguns modificam so a pele, outros a pele e os dentes, outros
// apenas os olhos, etc.
//
// Expressoes =================================================================
void Face::exibeExpressaoRaiva(double influencia) {
   pele->atualizaPoseManual(RAIVA, influencia);
   interiorBoca->atualizaPoseManual(RAIVA, influencia);
}

void Face::exibeExpressaoNojo(double influencia) {
   pele->atualizaPoseManual(NOJO, influencia);
}

void Face::exibeExpressaoMedo(double influencia) {
   pele->atualizaPoseManual(MEDO, influencia);
   lingua->atualizaPoseManual(MEDO, influencia);
   denteBaixo->atualizaPoseManual(MEDO, influencia);
   interiorBoca->atualizaPoseManual(MEDO, influencia);
}

void Face::exibeExpressaoTristeza(double influencia) {
   pele->atualizaPoseManual(TRISTEZA, influencia);
}

void Face::exibeExpressaoSorrisoFechado(double influencia) {
   pele->atualizaPoseManual(SORRISO_FECHADO, influencia);
}

void Face::exibeExpressaoSorrisoAberto(double influencia) {
   pele->atualizaPoseManual(SORRISO_ABERTO, influencia);
}

void Face::exibeExpressaoSurpresa(double influencia) {
   pele->atualizaPoseManual(SURPRESA, influencia);
}

// Visemas ====================================================================
void Face::exibeVisemaA(double influencia) {
   pele->atualizaPoseManual(VISEMA_A, influencia);
   lingua->atualizaPoseManual(VISEMA_A, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_A, influencia);
}

void Face::exibeVisemaE(double influencia) {
   pele->atualizaPoseManual(VISEMA_E, influencia);
   lingua->atualizaPoseManual(VISEMA_E, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_E, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_E, influencia);
}

void Face::exibeVisemaI(double influencia) {
   pele->atualizaPoseManual(VISEMA_I, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_I, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_I, influencia);
}

void Face::exibeVisemaO(double influencia) {
   pele->atualizaPoseManual(VISEMA_O, influencia);
   lingua->atualizaPoseManual(VISEMA_O, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_O, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_O, influencia);
}

void Face::exibeVisemaU(double influencia) {
   pele->atualizaPoseManual(VISEMA_U, influencia);
   lingua->atualizaPoseManual(VISEMA_U, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_U, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_U, influencia);
}

void Face::exibeVisemaBMP(double influencia) {
   pele->atualizaPoseManual(VISEMA_BMP, influencia);
}

void Face::exibeVisemaJX(double influencia) {
   pele->atualizaPoseManual(VISEMA_JX, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_JX, influencia);
}

void Face::exibeVisemaDST(double influencia) {
   pele->atualizaPoseManual(VISEMA_DST, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_DST, influencia);
}

void Face::exibeVisemaFV(double influencia) {
   pele->atualizaPoseManual(VISEMA_FV, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_FV, influencia);
}

void Face::exibeVisemaKG(double influencia) {
   pele->atualizaPoseManual(VISEMA_KG, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_KG, influencia);
}

void Face::exibeVisemaLN(double influencia) {
   pele->atualizaPoseManual(VISEMA_LN, influencia);
   lingua->atualizaPoseManual(VISEMA_LN, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_LN, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_LN, influencia);
}

void Face::exibeVisemaR(double influencia) {
   pele->atualizaPoseManual(VISEMA_R, influencia);
   denteBaixo->atualizaPoseManual(VISEMA_R, influencia);
   interiorBoca->atualizaPoseManual(VISEMA_R, influencia);
}

// Acoes ======================================================================
void Face::exibeAcaoPiscarEsq(double influencia) {
   pele->atualizaPoseManual(PISCAR_ESQ, influencia);
}

void Face::exibeAcaoPiscarDir(double influencia) {
   pele->atualizaPoseManual(PISCAR_DIR, influencia);
}

void Face::exibeAcaoAbaixarSobrancelhaEsq(double influencia) {
   pele->atualizaPoseManual(ABAIXAR_SOBR_ESQ, influencia);
}

void Face::exibeAcaoAbaixarSobrancelhaDir(double influencia) {
   pele->atualizaPoseManual(ABAIXAR_SOBR_DIR, influencia);
}

void Face::exibeAcaoLevantarSobrancelhaEsq(double influencia) {
   pele->atualizaPoseManual(LEVANTAR_SOBR_ESQ, influencia);
}

void Face::exibeAcaoLevantarSobrancelhaDir(double influencia) {
   pele->atualizaPoseManual(LEVANTAR_SOBR_DIR, influencia);
}

void Face::exibeAcaoApertarOlhoEsq(double influencia) {
   pele->atualizaPoseManual(APERTAR_OLHO_ESQ, influencia);
}

void Face::exibeAcaoApertarOlhoDir(double influencia) {
   pele->atualizaPoseManual(APERTAR_OLHO_DIR, influencia);
}

// Movimento dos olhos ========================================================
void Face::exibeMovimentoOlharBaixo(double influencia) {
   olhoDireito->atualizaPoseManual(1, influencia);
   olhoEsquerdo->atualizaPoseManual(1, influencia);
   pele->atualizaPoseManual(OLHAR_BAIXO, influencia);
}

void Face::exibeMovimentoOlharCima(double influencia) {
   olhoDireito->atualizaPoseManual(2, influencia);
   olhoEsquerdo->atualizaPoseManual(2, influencia);
   pele->atualizaPoseManual(OLHAR_CIMA, influencia);
}

void Face::exibeMovimentoOlharEsq(double influencia) {
   olhoDireito->atualizaPoseManual(3, influencia);
   olhoEsquerdo->atualizaPoseManual(3, influencia);
}

void Face::exibeMovimentoOlharDir(double influencia) {
   olhoDireito->atualizaPoseManual(4, influencia);
   olhoEsquerdo->atualizaPoseManual(4, influencia);
}

// Acoes compostas ============================================================
void Face::exibeAcaoPiscar(double influencia) {
   exibeAcaoPiscarEsq(influencia);
   exibeAcaoPiscarDir(influencia);
}

void Face::exibeAcaoAbaixarSobrancelha(double influencia) {
   exibeAcaoAbaixarSobrancelhaEsq(influencia);
   exibeAcaoAbaixarSobrancelhaDir(influencia);
}

void Face::exibeAcaoLevantarSobrancelha(double influencia) {
   exibeAcaoLevantarSobrancelhaEsq(influencia);
   exibeAcaoLevantarSobrancelhaDir(influencia);
}

void Face::exibeAcaoApertarOlho(double influencia) {
   exibeAcaoApertarOlhoEsq(influencia);
   exibeAcaoApertarOlhoDir(influencia);
}

// Guarda a influencia de uma pose
void Face::armazenaInfluenciaPose(int pose, double valor)
{
   infPose[pose] = valor;
}

// Retorna a influencia de uma pose
double Face::devolveInfluenciaPose(int pose)
{
   return infPose[pose];
}

