#ifndef __FACE_HPP__
#define __FACE_HPP__

#include "ModeloFace.hpp"

// Numero de visemas e expressoes faciais
#define NUM_VIS 12
#define NUM_EXPR 8

#define NUM_POSES 36

// Define os indices para as poses disponiveis no arquivo da face
enum {
   NEUTRA, RAIVA, NOJO, MEDO, TRISTEZA,
   SORRISO_FECHADO, SORRISO_ABERTO, SURPRESA,
   VISEMA_A, VISEMA_BMP, VISEMA_DST, VISEMA_E, VISEMA_FV, VISEMA_I, 
   VISEMA_JX, VISEMA_KG, VISEMA_LN, VISEMA_O, VISEMA_R, VISEMA_U,
   PISCAR_ESQ, PISCAR_DIR, ABAIXAR_SOBR_ESQ, ABAIXAR_SOBR_DIR,
   LEVANTAR_SOBR_ESQ, LEVANTAR_SOBR_DIR, APERTAR_OLHO_ESQ, APERTAR_OLHO_DIR,
   OLHAR_BAIXO, OLHAR_CIMA, OLHAR_ESQ, OLHAR_DIR,
   PISCAR, ABAIXAR_SOBRANCELHA, LEVANTAR_SOBRANCELHA, APERTAR_OLHO
};

// Classe que manipula a face do programa, disponibilizando uma serie de 
// comandos de animacao, que modificam a malha ao longo do tempo
class Face : public ModeloFace
{
public:
   Face();
   ~Face() {};
   
   // Atualizacao das animacoes manuais e automaticas da face
   void atualizaAnimacaoManual(int acao, double influencia);
   void atualizaAnimacaoAutomatica(const FrameEvent &evt);

   // Armazena o valor atual da influencia que uma pose possui
   void armazenaInfluenciaPose(int pose, double valor);
   double devolveInfluenciaPose(int pose);

private:
   // Vetor que guarda as influencias
   double infPose[NUM_POSES];

   // Comandos que alteram a expressao facial do avatar
   void exibeExpressaoRaiva(double influencia);
   void exibeExpressaoNojo(double influencia);
   void exibeExpressaoMedo(double influencia);
   void exibeExpressaoTristeza(double influencia);
   void exibeExpressaoSorrisoFechado(double influencia);
   void exibeExpressaoSorrisoAberto(double influencia);
   void exibeExpressaoSurpresa(double influencia);

   // Comandos de exibicao de visemas
   void exibeVisemaA(double influencia);
   void exibeVisemaBMP(double influencia);
   void exibeVisemaDST(double influencia);
   void exibeVisemaE(double influencia);
   void exibeVisemaFV(double influencia);
   void exibeVisemaI(double influencia);
   void exibeVisemaJX(double influencia);
   void exibeVisemaKG(double influencia);
   void exibeVisemaLN(double influencia);
   void exibeVisemaO(double influencia);
   void exibeVisemaR(double influencia);
   void exibeVisemaU(double influencia);

   // Comandos de acoes que a face pode executar sobre cada um dos olhos e cada
   // uma das sobrancelhas
   void exibeAcaoPiscarEsq(double influencia);
   void exibeAcaoPiscarDir(double influencia);
   void exibeAcaoAbaixarSobrancelhaEsq(double influencia);
   void exibeAcaoAbaixarSobrancelhaDir(double influencia);
   void exibeAcaoLevantarSobrancelhaEsq(double influencia);
   void exibeAcaoLevantarSobrancelhaDir(double influencia);
   void exibeAcaoApertarOlhoEsq(double influencia);
   void exibeAcaoApertarOlhoDir(double influencia);
   
   // Comandos como os de cima, mas que deformam ambos os olhos e ambas as 
   // sobrancelhas
   void exibeAcaoPiscar(double influencia);
   void exibeAcaoAbaixarSobrancelha(double influencia);
   void exibeAcaoLevantarSobrancelha(double influencia);
   void exibeAcaoApertarOlho(double influencia);

   // Comados para movimentacao dos olhos do avatar
   void exibeMovimentoOlharBaixo(double influencia);
   void exibeMovimentoOlharCima(double influencia);
   void exibeMovimentoOlharEsq(double influencia);
   void exibeMovimentoOlharDir(double influencia);
};

#endif
