#include "FrameListenerAvator.hpp"

// Construtor
FrameListenerAvator::FrameListenerAvator(RenderWindow* jan, Face* face,
      CEGUI::Editbox* caixaDeTexto, bool usaCamera) 
   : FrameListenerCEGUI(jan)
{
   // Zera o contador de frames
   cntFrames = 0;

   // Manda o programa prosseguir
   continueRenderizando = true;

   // Define a face e a caixa de texto da aplicacao
   this->face = face;
   this->caixaDeTexto = caixaDeTexto;

   this->usaCamera = usaCamera;

   // Cria os objetos de movimentacao
   idle = new MovimentacaoNaoVerbal(face);
   olhos = new MovimentacaoOlhos(face);
   fala = new TratamentoFala(face);
   expr = new TratamentoExpressao(face);
}

// Define os eventos que ocorrem ao iniciar um frame
bool FrameListenerAvator::frameStarted(const FrameEvent &evt)
{
   // Metodo da superclasse
   if (FrameListenerCEGUI::frameStarted(evt) == false)
      return false;

   // Recebe eventos de mouse e teclado
   if(mouse)
      mouse->capture();
   if(teclado)
      teclado->capture();

   // Animacao de fala e expresssoes faciais
   fala->trataMudancaVisema();
   expr->trataMudancaExpressao();

   // Animacao de movimentos dos olhos
   if (usaCamera)
      olhos->segueUsuario();
   else
      olhos->moveAleatoriamente(cntFrames);

   // Animacao de movimentos involuntarios
   idle->trataAcaoInvoluntaria(cntFrames);
   idle->trataRotacaoInvoluntaria();

   // Incrementa o contador de frames
   cntFrames++;

   // Zera o contador quando ele alcancar um valor muito grande
   if (cntFrames == 1000000)
      cntFrames = 0;

   // Segue a renderizacao
   return continueRenderizando;
}

// Especifica os eventos que ocorrem ao final de um frame
bool FrameListenerAvator::frameEnded(const FrameEvent &evt)
{
   return true;
}

// Trata o evento de pressionar uma tecla
bool FrameListenerAvator::keyPressed(const OIS::KeyEvent &e)
{
   if (FrameListenerCEGUI::keyPressed(e) == false)
      return false;

   switch (e.key)
   {
      // A tecla ESC interrompe o programa
      case OIS::KC_ESCAPE:
         continueRenderizando = false;
         break;

      // Tecla ENTER
      case OIS::KC_RETURN:
         // Espera terminar de falar. O eSpeak resulta em erro quando duas 
         // frases sao pronunciadas ao mesmo tempo
         if (!Audio::getSingleton().estaFalando())
         {
            // Recebe o comando da caixa de texto
            String frase = (caixaDeTexto->getText()).c_str();
            StringUtil::toUpperCase(frase);

            if (frase.length() > 0)
            {
               // Processa o comando, devolvendo a acao que a face deve executar
               if (usaCamera && Visao::getSingleton().usuarioChegou()
                     || !usaCamera)
               {
                  String conteudoFala = fala->processaEntrada(frase, expr);
                  fala->reproduzFala(conteudoFala);
               }
               else
                  fala->reproduzFala("ONDE ESTÁ VOCÊ?");
            }
            // Apaga o conteudo da caixa de texto
            caixaDeTexto->setText("");
         }
         break;
   }
   return true;
}

