#include "FrameListenerBase.hpp"

// Construtor
FrameListenerBase::FrameListenerBase(RenderWindow* jan, bool bufferedKeys, 
      bool bufferedMouse) 
   : janela(jan), inputMgr(0), mouse(0), teclado(0)
{
   using namespace OIS;

   // Lista de parametros para configuracao do OIS
   ParamList lista;
   size_t windowHnd = 0;
   std::ostringstream windowHndStr;

   // Inicializacao do OIS
   LogManager::getSingletonPtr()->logMessage("*** Initializing OIS ***");

   jan->getCustomAttribute("WINDOW", &windowHnd);
   windowHndStr << windowHnd;
   lista.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));

   inputMgr = InputManager::createInputSystem(lista);

   // Cria dispositivos de teclado e mouse usando buffered input
   teclado = static_cast<Keyboard*>(inputMgr->createInputObject(OISKeyboard, 
            bufferedKeys));
   mouse = static_cast<Mouse*>(inputMgr->createInputObject(OISMouse, 
            bufferedMouse));

   // Ajusta area do mouse
   redimensiona(janela);

   // Registra o Window listener
   WindowEventUtilities::addWindowEventListener(janela, this);
}

// Destrutor
FrameListenerBase::~FrameListenerBase()
{
   // Remove o Window listener
   WindowEventUtilities::removeWindowEventListener(janela, this);
   encerra(janela);
}

// Ajusta area do mouse para que respeite os limites da janela
void FrameListenerBase::redimensiona(RenderWindow* janela)
{
   int esq, cima;
   unsigned int altura, largura, prof;

   // Recebe as dimensoes da janela
   janela->getMetrics(largura, altura, prof, esq, cima);

   // Define a area do mouse de acordo
   const OIS::MouseState &ms = mouse->getMouseState();
   ms.height = altura;
   ms.width = largura;
}

// Encerra OIS antes de fechar a janela (fundamental no linux)
void FrameListenerBase::encerra(RenderWindow* janela)
{
   // Fecha a janela que criou o OIS
   if (this->janela == janela)
   {
      if (inputMgr)
      {
         // Encerra os dispositivos
         inputMgr->destroyInputObject(mouse);
         inputMgr->destroyInputObject(teclado);

         OIS::InputManager::destroyInputSystem(inputMgr);
         inputMgr = 0;
      }
   }
}


