#include "FrameListenerCEGUI.hpp"

// Construtor
FrameListenerCEGUI::FrameListenerCEGUI(RenderWindow* jan)
   : FrameListenerBase(jan, true, true)
{
   mouse->setEventCallback(this);
   teclado->setEventCallback(this);
}

// Converte o botao do mouse do OIS para um botao do CEGUI
CEGUI::MouseButton FrameListenerCEGUI::convertOISButtonToCegui(int buttonID)
{
   using namespace OIS; 

   switch (buttonID)
   {
      case OIS::MB_Left:
         return CEGUI::LeftButton;
      case OIS::MB_Right:
         return CEGUI::RightButton;
      case OIS::MB_Middle:
         return CEGUI::MiddleButton;
      default:
         return CEGUI::LeftButton;
   }
}

// Inicio do frame
bool FrameListenerCEGUI::frameStarted(const FrameEvent &evt)
{ 
   CEGUI::System::getSingleton().injectTimePulse(static_cast<float>
         (evt.timeSinceLastFrame));
   return true;
}

// Final do frame
bool FrameListenerCEGUI::frameEnded(const FrameEvent &evt)
{
   return true;
}

// Trata evento de mover o mouse
bool FrameListenerCEGUI::mouseMoved(const OIS::MouseEvent &e)
{
   CEGUI::System::getSingleton().injectMouseMove(e.state.X.rel, e.state.Y.rel);
   return true;
}

// Trata evento de pressionar o botao do mouse
bool FrameListenerCEGUI::mousePressed(const OIS::MouseEvent &e, 
      OIS::MouseButtonID id)
{
   CEGUI::System::getSingleton().injectMouseButtonDown
      (convertOISButtonToCegui(id));
   return true;
}

// Trata evento de soltar o botao do mouse
bool FrameListenerCEGUI::mouseReleased(const OIS::MouseEvent &e, 
      OIS::MouseButtonID id)
{
   CEGUI::System::getSingleton().injectMouseButtonUp
      (convertOISButtonToCegui(id));
   return true;
}

// Trata evento de pressionar uma tecla
bool FrameListenerCEGUI::keyPressed(const OIS::KeyEvent &e)
{
   CEGUI::System::getSingleton().injectKeyDown(e.key);
   CEGUI::System::getSingleton().injectChar(e.text);
   return true;
}

// Trata evento de soltar uma tecla
bool FrameListenerCEGUI::keyReleased(const OIS::KeyEvent &e)
{
   CEGUI::System::getSingleton().injectKeyUp(e.key);
   return true;
}



