#include "ModeloAnimado.hpp"

// Cria a animacao manual do objeto
void ModeloAnimado::criaAnimacaoManual()
{
   // Carrega o arquivo da malha
   MeshPtr malha = MeshManager::getSingleton().load(arquivoMalha, 
         ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);

   // Cria a animacao manual
   Animation* animacao = malha->createAnimation("Manual", 0);

   // A animacao modifica os vertices do objeto
   VertexAnimationTrack* track = animacao->createVertexTrack(1, VAT_POSE);
   verticeChave = track->createVertexPoseKeyFrame(0);

   // Armazena o numero de poses presente no arquivo da malha
   numPoses = malha->getPoseCount();

   // Inicialmente, todas as poses sao adicionadas com influencia zero
   for (int i = 0; i < numPoses; i++)
      verticeChave->addPoseReference(i, 0.0f);
}

// Inicializa a animacao manual criada
void ModeloAnimado::inicializaAnimacaoManual()
{
   animacaoManual = entidade->getAnimationState("Manual");
   animacaoManual->setTimePosition(0);

   // Habilita
   animacaoManual->setEnabled(true);
}

// Remove a animacao manual criada
void ModeloAnimado::removeAnimacaoManual()
{
   // Carrega o arquivo da malha
   MeshPtr malha = MeshManager::getSingleton().load(arquivoMalha, 
         ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME);

   // Remove a animacao manual
   Animation* animacao = malha->getAnimation("Manual");
   malha->removeAnimation("Manual");
   animacao->destroyVertexTrack(1);
   verticeChave->removeAllPoseReferences();

   // Desabilita
   animacaoManual->setEnabled(false);
}

// Atualiza a animacao, atualizando a influencia da pose
void ModeloAnimado::atualizaPoseManual(int indice, double influencia)
{
   // Modifica a pose do indice passado para possuir a influencia especificada
   verticeChave->updatePoseReference(indice, influencia);
   animacaoManual->getParent()->_notifyDirty();
}

// Inicializa a animacao automatica presente no arquivo da malha
void ModeloAnimado::inicializaAnimacaoAutomatica()
{
   // Os arquivos .mesh que acompanham o programa possuem uma animacao nomeada
   // "All" que dispoe todas as poses
   animacaoAutomatica = entidade->getAnimationState("All");
   animacaoAutomatica->setEnabled(true);
}

// Atualiza a animacao automatica, a partir do tempo do frame anterior
void ModeloAnimado::atualizaPoseAutomatica(const FrameEvent &evt)
{
   animacaoAutomatica->addTime(evt.timeSinceLastFrame);
}

// Retorna o numero de poses presentes no arquivo mesh do modelo
int ModeloAnimado::getNumPoses()
{
   return numPoses;
}
