#ifndef __MODELOANIMADO_HPP__
#define __MODELOANIMADO_HPP__

#include "Ogre.h"
#include "Modelo.hpp"

using namespace Ogre;

// Cria um modelo tridimensional que possui animacao manual e automatica
// A face e composta por 8 submalhas, sendo 7 modelos animados e 1 modelo 
// estatico (o do cabelo)
class ModeloAnimado : public Modelo
{
public:
   ModeloAnimado(String arquivoMalha)
      : Modelo (arquivoMalha) {};
   ~ModeloAnimado() {};

   // Atualiza as poses
   void atualizaPoseManual(int indice, double influencia);
   void atualizaPoseAutomatica(const FrameEvent &evt);

   // Configuracao das animacoes do modelo
   void criaAnimacaoManual();
   void inicializaAnimacaoManual();
   void removeAnimacaoManual();
   void inicializaAnimacaoAutomatica();
   
   // Devolve o numero de poses
   int getNumPoses();
   
private:
   // Numero de poses no arquivo mesh
   int numPoses;

   // Animacoes automatica e manual
   AnimationState* animacaoAutomatica;
   AnimationState* animacaoManual;

   // Vertice chave que representa uma pose definida no arquivo
   VertexPoseKeyFrame* verticeChave;
};

#endif
