#include "ModeloFace.hpp"

// Inicializa cada componente da face
void ModeloFace::carregaComponentesFace()
{
   // Cria cada componente da face
   olhoEsquerdo = new ModeloAnimado(nomeAvatar + "_eyeL.mesh");
   olhoDireito = new ModeloAnimado(nomeAvatar + "_eyeR.mesh");
   pele = new ModeloAnimado(nomeAvatar + "_skin.mesh");
   denteBaixo = new ModeloAnimado(nomeAvatar + "_teethLower.mesh");
   denteCima = new ModeloAnimado(nomeAvatar + "_teethUpper.mesh");
   lingua = new ModeloAnimado(nomeAvatar + "_tongue.mesh");
   interiorBoca = new ModeloAnimado(nomeAvatar + "_sock.mesh");
   cabelo = new Modelo(nomeAvatar + "_hair.mesh");
}

// Seleciona cada componente da face
void ModeloFace::recebeComponentesFace(SceneManager* sceneMgr)
{
   olhoEsquerdo->setEntidade(sceneMgr->getEntity(nomeAvatar + "_eyeL.mesh"));
   olhoDireito->setEntidade(sceneMgr->getEntity(nomeAvatar + "_eyeR.mesh"));
   pele->setEntidade(sceneMgr->getEntity(nomeAvatar + "_skin.mesh"));
   denteBaixo->setEntidade(sceneMgr->getEntity(nomeAvatar +"_teethLower.mesh"));
   denteCima->setEntidade(sceneMgr->getEntity(nomeAvatar + "_teethUpper.mesh"));
   lingua->setEntidade(sceneMgr->getEntity(nomeAvatar + "_tongue.mesh"));
   interiorBoca->setEntidade(sceneMgr->getEntity(nomeAvatar + "_sock.mesh"));
   cabelo->setEntidade(sceneMgr->getEntity(nomeAvatar + "_hair.mesh"));
}

// Cria cada uma das entidades
void ModeloFace::criaEntidadesFace(SceneManager* sceneMgr)
{
   olhoEsquerdo->criaEntidade(sceneMgr);
   olhoDireito->criaEntidade(sceneMgr);
   pele->criaEntidade(sceneMgr);
   denteBaixo->criaEntidade(sceneMgr);
   denteCima->criaEntidade(sceneMgr);
   lingua->criaEntidade(sceneMgr);
   interiorBoca->criaEntidade(sceneMgr);
   cabelo->criaEntidade(sceneMgr);
}

// Cria as animacoes manuais para cada componente
void ModeloFace::criaAnimacoesFace()
{
   olhoEsquerdo->criaAnimacaoManual();
   olhoDireito->criaAnimacaoManual();
   pele->criaAnimacaoManual();
   denteBaixo->criaAnimacaoManual();
   denteCima->criaAnimacaoManual();
   lingua->criaAnimacaoManual();
   interiorBoca->criaAnimacaoManual();
}

// Inicializa as animacoes manuais da face
void ModeloFace::inicializaAnimacoesFace()
{
   olhoEsquerdo->inicializaAnimacaoManual();
   olhoDireito->inicializaAnimacaoManual();
   pele->inicializaAnimacaoManual();
   denteBaixo->inicializaAnimacaoManual();
   denteCima->inicializaAnimacaoManual();
   lingua->inicializaAnimacaoManual();
   interiorBoca->inicializaAnimacaoManual();
}

// Remove as animacoes manuais da face
void ModeloFace::removeAnimacoesFace()
{
   olhoEsquerdo->removeAnimacaoManual();
   olhoDireito->removeAnimacaoManual();
   pele->removeAnimacaoManual();
   denteBaixo->removeAnimacaoManual();
   denteCima->removeAnimacaoManual();
   lingua->removeAnimacaoManual();
   interiorBoca->removeAnimacaoManual();
}

// Insere o no da face na cena, a partir da posicao especificada
void ModeloFace::criaNoFace(SceneManager* sceneMgr, Vector3 pos)
{
   noFace = sceneMgr->getRootSceneNode()->createChildSceneNode("NoFace", pos);
}

// Insere as entidades no no da face
void ModeloFace::anexaFaceCena()
{
   olhoEsquerdo->anexaEntidade(noFace);
   olhoDireito->anexaEntidade(noFace);
   pele->anexaEntidade(noFace);
   denteBaixo->anexaEntidade(noFace);
   denteCima->anexaEntidade(noFace);
   lingua->anexaEntidade(noFace);
   interiorBoca->anexaEntidade(noFace);
   cabelo->anexaEntidade(noFace);
}

// Remove as entidades do no da face
void ModeloFace::removeFaceCena(SceneManager* sceneMgr)
{
   olhoEsquerdo->removeEntidade(sceneMgr, noFace);
   olhoDireito->removeEntidade(sceneMgr, noFace);
   pele->removeEntidade(sceneMgr, noFace);
   denteBaixo->removeEntidade(sceneMgr, noFace);
   denteCima->removeEntidade(sceneMgr, noFace);
   lingua->removeEntidade(sceneMgr, noFace);
   interiorBoca->removeEntidade(sceneMgr, noFace);
   cabelo->removeEntidade(sceneMgr, noFace);
}

// Rotaciona o no da face no eixo passado por parametro
void ModeloFace::rotaciona(char eixo, double angulo)
{
   if (eixo == 'X')
      noFace->pitch(Degree(angulo));

   else if (eixo == 'Y')
      noFace->yaw(Degree(angulo));

   else if (eixo == 'Z')
      noFace->roll(Degree(angulo));
}

// Define um nome para a face
void ModeloFace::setNome(String nome)
{
   // Todo avatar possui um nome, que serve tambem como prefixo para os arquivos
   // das malhas
   nomeAvatar = nome;
}

// Devolve o nome da face
String ModeloFace::getNome()
{
   return nomeAvatar;
}

