#include "MovimentacaoNaoVerbal.hpp"

// Construtor: inicializa as animacoes
MovimentacaoNaoVerbal::MovimentacaoNaoVerbal(Face* face)
{
   this->face = face;

   for (int i = 0; i < NUM_ACOES; i++)
      animAcao[i] = new AnimacaoPose(face);

   for (int i = 0; i < NUM_ROTACOES; i++)
      animRotacao[i] = new AnimacaoRotacao(face);
}

// Dispara, em tempos aleatorios, as acoes involuntarias do avatar
void MovimentacaoNaoVerbal::trataAcaoInvoluntaria(int cntFrames)
{
   // Pisca
   animAcao[0]->sorteiaNovoTempo(cntFrames, 300, 200);
   animAcao[0]->interpolaDesfaz(PISCAR, infPisca, 0.03);

   // Levanta a sobrancelha
   animAcao[1]->sorteiaNovoTempo(cntFrames, 800, 500);
   animAcao[1]->interpolaDesfaz(LEVANTAR_SOBRANCELHA, 1.0, 0.01);

   // Aperta os olhos
   animAcao[2]->sorteiaNovoTempo(cntFrames, 1000, 500);
   animAcao[2]->interpolaDesfaz(APERTAR_OLHO, 1.0, 0.01);

   // Sorri, com influencia bem pequena, apenas mexendo os cantos da boca
   animAcao[3]->sorteiaNovoTempo(cntFrames, 1000, 500);
   animAcao[3]->interpolaDesfaz(SORRISO_FECHADO, 0.3, 0.01);

   // Na expressoes de raiva, nojo e tristeza a face apresenta os olhos
   // apertados. Diminui a influencia da acao de piscar nestes casos
   if (face->devolveInfluenciaPose(RAIVA) == INF_EXPR
         || face->devolveInfluenciaPose(NOJO) == INF_EXPR
         || face->devolveInfluenciaPose(TRISTEZA) == INF_EXPR)
      infPisca = 0.7;
   else
      infPisca = 1.0;
}

// Rotacoes com velocidades aleatorias
void MovimentacaoNaoVerbal::trataRotacaoInvoluntaria()
{
   // Ao redor do eixo x
   animRotacao[0]->sorteiaNovaVelRotacao(VELX);
   animRotacao[0]->rotacionaDesfaz('X', ANGULOX);

   // Ao redor do eixo y
   animRotacao[1]->sorteiaNovaVelRotacao(VELY);
   animRotacao[1]->rotacionaDesfaz('Y', ANGULOY);

   // Ao redor do eixo z
   animRotacao[2]->sorteiaNovaVelRotacao(VELZ);
   animRotacao[2]->rotacionaDesfaz('Z', ANGULOZ);
}

