#include "MovimentacaoOlhos.hpp"

// Construtor: inicializa as animacoes
MovimentacaoOlhos::MovimentacaoOlhos(Face* face)
{
   for (int i = 0; i < 4; i++)
      animOlhos[i] = new AnimacaoPose(face);
}

// Movimenta os olhos conforme a posicao da face do usuario
void MovimentacaoOlhos::segueUsuario()
{
   // Se a face do usuario foi detectada
   if (Visao::getSingleton().usuarioChegou())
   {
      // Pega a coordenada em x da face
      int posX = Visao::getSingleton().getX();
      // Pega a coordenada em y da face
      int posY = Visao::getSingleton().getY();

      // Se a face estiver a direita da tela, os olhos se movem para a direita,
      // com influencia de acordo com a posicao relativa da face
      if (posX < CAM_X)
      {
         animOlhos[0]->atualizaPose(OLHAR_DIR, (double)(CAM_X-posX)/(2*CAM_X));
         animOlhos[1]->atualizaPose(OLHAR_ESQ, 0);
      }
      // Senao, os olhos se movem para a esquerda, analogamente
      else
      {
         animOlhos[1]->atualizaPose(OLHAR_ESQ, (double)(posX-CAM_X)/(2*CAM_X));
         animOlhos[0]->atualizaPose(OLHAR_DIR, 0);
      }

      // Se a face estiver na porcao de cima da tela, os olhos se movem para 
      // cima, com influencia de acordo com a posicao relativa da face
      if (posY < CAM_Y)
      {
         animOlhos[2]->atualizaPose(OLHAR_CIMA,(double)(CAM_Y-posY)/(2*CAM_Y));
         animOlhos[3]->atualizaPose(OLHAR_BAIXO, 0);
      }
      // Senao, os olhos se movem para baixo, analogamente
      else
      {
         animOlhos[3]->atualizaPose(OLHAR_BAIXO,(double)(posY-CAM_Y)/(2*CAM_Y));
         animOlhos[2]->atualizaPose(OLHAR_CIMA, 0);
      }
   }
}

// Movimenta os olhos aleatoriamente, nas quatro direcoes
void MovimentacaoOlhos::moveAleatoriamente(int cntFrames)
{
   // Move os olhos, aleatoriamente, para direita
   if (animOlhos[0]->encerrouAnimacao())
   {
      animOlhos[1]->sorteiaNovoTempo(cntFrames, 700, 500);
      animOlhos[1]->interpolaDesfaz(OLHAR_DIR, INF_OLHOS, VEL_OLHOS);
   }
   // Move os olhos, aleatoriamente, para esquerda
   if (animOlhos[1]->encerrouAnimacao())
   {
      animOlhos[0]->sorteiaNovoTempo(cntFrames, 500, 500);
      animOlhos[0]->interpolaDesfaz(OLHAR_ESQ, INF_OLHOS, VEL_OLHOS);
   }
   // Move os olhos, aleatoriamente, para cima
   if (animOlhos[2]->encerrouAnimacao())
   {
      animOlhos[3]->sorteiaNovoTempo(cntFrames, 600, 500);
      animOlhos[3]->interpolaDesfaz(OLHAR_CIMA, INF_OLHOS, VEL_OLHOS);
   }
   // Move os olhos, aleatoriamente, para baixo
   if (animOlhos[3]->encerrouAnimacao())
   {
      animOlhos[2]->sorteiaNovoTempo(cntFrames, 800, 500);
      animOlhos[2]->interpolaDesfaz(OLHAR_BAIXO, INF_OLHOS, VEL_OLHOS);
   }
}

