#include "SincronizacaoLabial.hpp"

// Construtor
SincronizacaoLabial::SincronizacaoLabial()
{
   montaTabelaVisemas();
}

// Destrutor
SincronizacaoLabial::~SincronizacaoLabial()
{
   tabela.clear();
}

// Metodo que sincroniza fonemas e visemas
void SincronizacaoLabial::sincroniza(AnimacaoPose* anim[])
{
   if (!Audio::getSingleton().estaFalando())
      fonemaAnterior = '$';

   else
   {
      // Recebe o fonema que esta sendo enunciado no momento
      const char* fonema = Audio::getSingleton().fonemaAtual();

      // Percorre a tabela de visemas
      for (multimap<int,char>::iterator i=tabela.begin(); i!=tabela.end(); ++i)
      {
         // Considera apenas o primeiro caractere da string do fonema, pois
         // marcadores do tipo ~, /, -, etc, nao influenciam no visema
         if (fonema[0] == (*i).second)
         {
            if (fonema[0] != fonemaAnterior)
            {
               // De acordo com o fonema recebido, uma animacao de exibicao de 
               // visema e disparada
               anim[(*i).first - NUM_EXPR]->iniciaAnimacao(0);

               // Fonemas aI, aU, eI, eU, oI, iU, uI
               if (fonema[1] == 'I')
                  anim[VISEMA_I - NUM_EXPR]->iniciaAnimacao(0);
               else if (fonema[1] == 'U')
                  anim[VISEMA_U - NUM_EXPR]->iniciaAnimacao(0);

               fonemaAnterior = fonema[0];
            }
            break;
         }
      }
   }
}

// Determina os grupos de fonemas ao qual cada visema ira corresponder
void SincronizacaoLabial::montaTabelaVisemas()
{
   // Estes fonemas sao inseridos na tabela, tendo como chave o visema 
   // correspondente
   tabela.insert(pair<int, char>(VISEMA_A, 'a'));
   tabela.insert(pair<int, char>(VISEMA_A, 'A'));
   tabela.insert(pair<int, char>(VISEMA_A, '&'));

   tabela.insert(pair<int, char>(VISEMA_E, 'e'));
   tabela.insert(pair<int, char>(VISEMA_E, 'E'));

   tabela.insert(pair<int, char>(VISEMA_I, 'i'));
   tabela.insert(pair<int, char>(VISEMA_I, 'j'));

   tabela.insert(pair<int, char>(VISEMA_O, 'o'));
   tabela.insert(pair<int, char>(VISEMA_O, 'O'));

   tabela.insert(pair<int, char>(VISEMA_U, 'u'));
   tabela.insert(pair<int, char>(VISEMA_U, 'U'));
   tabela.insert(pair<int, char>(VISEMA_U, 'w'));

   tabela.insert(pair<int, char>(VISEMA_BMP, 'b'));
   tabela.insert(pair<int, char>(VISEMA_BMP, 'm'));
   tabela.insert(pair<int, char>(VISEMA_BMP, 'p'));

   tabela.insert(pair<int, char>(VISEMA_DST, 'd'));
   tabela.insert(pair<int, char>(VISEMA_DST, 's'));
   tabela.insert(pair<int, char>(VISEMA_DST, 't'));
   tabela.insert(pair<int, char>(VISEMA_DST, 'T'));
   tabela.insert(pair<int, char>(VISEMA_DST, 'z'));

   tabela.insert(pair<int, char>(VISEMA_FV, 'f'));
   tabela.insert(pair<int, char>(VISEMA_FV, 'v'));

   tabela.insert(pair<int, char>(VISEMA_JX, 'S'));
   tabela.insert(pair<int, char>(VISEMA_JX, 'Z'));

   tabela.insert(pair<int, char>(VISEMA_KG, 'k'));
   tabela.insert(pair<int, char>(VISEMA_KG, 'g'));

   tabela.insert(pair<int, char>(VISEMA_LN, 'l'));
   tabela.insert(pair<int, char>(VISEMA_LN, 'n'));

   tabela.insert(pair<int, char>(VISEMA_R, 'r'));
   tabela.insert(pair<int, char>(VISEMA_R, 'R'));
   tabela.insert(pair<int, char>(VISEMA_R, 'x'));
}


