#include "TratamentoExpressao.hpp"

// Construtor: inicializa as animacoes
TratamentoExpressao::TratamentoExpressao(Face* face)
{
   this->face = face;

   for (int i = 0; i < NUM_EXPR; i++)
   {
      animIni[i] = new AnimacaoPose(face);
      animFim[i] = new AnimacaoPose(face);
   }
}

// Tratamento das expressoes faciais
void TratamentoExpressao::trataMudancaExpressao()
{
   // Muda expressao antes de pronunciar o texto
   animIni[RAIVA]->interpola(exprIni[RAIVA], INF_EXPR, VEL_EXPR);
   animIni[NOJO]->interpola(exprIni[NOJO], INF_EXPR, VEL_EXPR);
   animIni[MEDO]->interpola(exprIni[MEDO], INF_EXPR, VEL_EXPR);
   animIni[TRISTEZA]->interpola(exprIni[TRISTEZA], INF_EXPR, VEL_EXPR);
   animIni[SORRISO_ABERTO]->interpola(exprIni[SORRISO_ABERTO], INF_EXPR, 
         VEL_EXPR);
   animIni[SURPRESA]->interpola(exprIni[SURPRESA], INF_EXPR, VEL_EXPR);

   // Muda expressao apos terminar de pronunciar o texto
   if (!Audio::getSingleton().estaFalando())
   {
      animFim[RAIVA]->interpola(exprFim[RAIVA], INF_EXPR, VEL_EXPR);
      animFim[NOJO]->interpola(exprFim[NOJO], INF_EXPR, VEL_EXPR);
      animFim[MEDO]->interpola(exprFim[MEDO], INF_EXPR, VEL_EXPR);
      animFim[TRISTEZA]->interpola(exprFim[TRISTEZA], INF_EXPR, VEL_EXPR);
      animFim[SORRISO_ABERTO]->interpola(exprFim[SORRISO_ABERTO], INF_EXPR, 
            VEL_EXPR);
      animFim[SURPRESA]->interpola(exprFim[SURPRESA], INF_EXPR, VEL_EXPR);
   }
}

// Exibe ou desfaz uma expressao facial no inicio da fala
void TratamentoExpressao::mudaExpressaoInicio(int acao, bool desfaz)
{
   if (!desfaz)
   {
      // Se a pose nao estiver sendo exibida, inicia a animacao
      if (face->devolveInfluenciaPose(acao) < INF_EXPR)
      {
         exprIni[acao] = acao;
         animIni[acao]->iniciaAnimacao(0);
      }
   }
   else
   {
      // Se a pose estiver sendo exibida, desfaz
      if (face->devolveInfluenciaPose(acao) > 0)
      {
         exprIni[acao] = -acao;
         animIni[acao]->iniciaAnimacao(INF_EXPR);
      }
   }
}

// Exibe ou desfaz uma expressao facial no final da fala
void TratamentoExpressao::mudaExpressaoFim(int acao, bool desfaz)
{
   if (!desfaz)
   {
      // Se a pose nao estiver sendo exibida, inicia a animacao
      if (face->devolveInfluenciaPose(acao) < INF_EXPR)
      {
         exprFim[acao] = acao;
         animFim[acao]->iniciaAnimacao(0);
      }
   }
   else
   {
      // Se a pose estiver sendo exibida, desfaz
      if (face->devolveInfluenciaPose(acao) > 0 || exprIni[acao] == acao)
      {
         exprFim[acao] = -acao;
         animFim[acao]->iniciaAnimacao(INF_EXPR);
      }
   }
}

// Zera o vetor, quando nao ha expressoes para serem exibidas
void TratamentoExpressao::zeraVetorExpressoes()
{
   for (int i = 0; i < NUM_EXPR; i++)
      exprIni[i] = exprFim[i] = 0;
}

