#include "TratamentoFala.hpp"

String fala;

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
//TODO Implementacao de thread no windows

// Implementacao de thread no linux
#else
pthread_t som;

// Cria uma thread para a fala
void* threadVoz(void* param)
{
   // Marca o inicio da fala
   Audio::getSingleton().iniciaFala();

   // Pronuncia
   const char* texto = fala.c_str();
   Audio::getSingleton().pronuncia(texto);
   espeak_Synchronize();

   // Marca o final da fala
   Audio::getSingleton().encerraFala();
}
#endif

// Construtor: inicializa as animacoes
TratamentoFala::TratamentoFala(Face* face)
{
   for (int i = 0; i < NUM_VIS; i++)
      animVisemas[i] = new AnimacaoPose(face);

   // Cria o objeto de sincronizacao
   labios = new SincronizacaoLabial();
}

// Reproduz o texto que deve ser falado
void TratamentoFala::reproduzFala(String frase)
{
   fala = frase;

   // Palavras que devem ser pronunciadas pela face
   if (fala.length() > 0)
   {
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
      // TODO TratamentoFala no windows
#else
      pthread_create(&som, NULL, threadVoz, NULL);
#endif
   }
}

// Tratamento dos visemas
void TratamentoFala::trataMudancaVisema()
{
   // Sincroniza
   labios->sincroniza(animVisemas);

   // Visemas possiveis que podem ser exibidos durante a fala
   if (Audio::getSingleton().estaFalando())
   {
      animVisemas[0]->interpolaDesfaz(VISEMA_A, INF_VIS, VEL_VIS);
      animVisemas[1]->interpolaDesfaz(VISEMA_BMP, INF_VIS, VEL_VIS);
      animVisemas[2]->interpolaDesfaz(VISEMA_DST, INF_VIS, VEL_VIS);
      animVisemas[3]->interpolaDesfaz(VISEMA_E, INF_VIS, VEL_VIS);
      animVisemas[4]->interpolaDesfaz(VISEMA_FV, INF_VIS, VEL_VIS);
      animVisemas[5]->interpolaDesfaz(VISEMA_JX, INF_VIS, VEL_VIS);
      animVisemas[6]->interpolaDesfaz(VISEMA_I, INF_VIS, VEL_VIS);
      animVisemas[7]->interpolaDesfaz(VISEMA_KG, INF_VIS, VEL_VIS);
      animVisemas[8]->interpolaDesfaz(VISEMA_LN, INF_VIS, VEL_VIS);
      animVisemas[9]->interpolaDesfaz(VISEMA_O, INF_VIS, VEL_VIS);
      animVisemas[10]->interpolaDesfaz(VISEMA_R, INF_VIS, VEL_VIS);
      animVisemas[11]->interpolaDesfaz(VISEMA_U, INF_VIS, VEL_VIS);
   }
}

// Trata o texto a ser falado, retirando possiveis marcacoes de mudanca de
// expressao facial. Ativa essas animacoes e pronuncia o restante do texto
String TratamentoFala::processaEntrada(String frase, TratamentoExpressao* expr)
{
   int acao;
   bool desfaz;

   fala = frase;

   expr->zeraVetorExpressoes();

   // Verifica se existem marcacoes de mudanca de expressao no inicio da frase
   while (1)
   {
      // Se existe uma tag para desfazer uma expressao
      if (StringUtil::startsWith(fala, "</", true))
      {
         fala = fala.erase(0, 2);
         desfaz = true;
      }
      // Se existe uma tag para exibir uma expressao
      else if (StringUtil::startsWith(fala, "<", true))
      {
         fala = fala.erase(0, 1);
         desfaz = false;
      }
      else
         break;

      // Expressoes
      if (StringUtil::startsWith(fala, "raiva>", true)) {
         // Apaga o marcador da frase pois ele nao sera pronunciado
         fala = fala.erase(0, 6);
         acao = RAIVA;
      }
      else if (StringUtil::startsWith(fala, "nojo>", true)) {
         fala = fala.erase(0, 5);
         acao = NOJO;
      }
      else if (StringUtil::startsWith(fala, "medo>", true)) {
         fala = fala.erase(0, 5);
         acao = MEDO;
      }
      else if (StringUtil::startsWith(fala, "tristeza>", true)) {
         fala = fala.erase(0, 9);
         acao = TRISTEZA;
      }
      else if (StringUtil::startsWith(fala, "alegria>", true)) {
         fala = fala.erase(0, 8);
         acao = SORRISO_ABERTO;
      }
      else if (StringUtil::startsWith(fala, "surpresa>", true)) {
         fala = fala.erase(0, 9);
         acao = SURPRESA;
      }
      else
         break;

      expr->mudaExpressaoInicio(acao, desfaz);
   }

   // Verifica se existem marcacoes de mudanca de expressao no final da frase
   while (1)
   {
      // Expressoes
      if (StringUtil::endsWith(fala, "raiva>", true)) {
         fala = fala.substr(0, fala.length() - 6);
         acao = RAIVA;
      }
      else if (StringUtil::endsWith(fala, "nojo>", true)) {
         fala = fala.substr(0, fala.length() - 5);
         acao = NOJO;
      }
      else if (StringUtil::endsWith(fala, "medo>", true)) {
         fala = fala.substr(0, fala.length() - 5);
         acao = MEDO;
      }
      else if (StringUtil::endsWith(fala, "tristeza>", true)) {
         fala = fala.substr(0, fala.length() - 9);
         acao = TRISTEZA;
      }
      else if (StringUtil::endsWith(fala, "alegria>", true)) {
         fala = fala.substr(0, fala.length() - 8);
         acao = SORRISO_ABERTO;
      }
      else if (StringUtil::endsWith(fala, "surpresa>", true)) {
         fala = fala.substr(0, fala.length() - 9);
         acao = SURPRESA;
      }
      // Senao, retorna
      else
         break;

      // Se existe uma tag para desfazer uma expressao
      if (StringUtil::endsWith(fala, "</", true))
      {
         fala = fala.substr(0, fala.length() - 2);
         desfaz = true;
      }
      // Se existe uma tag para exibir uma expressao
      else if (StringUtil::endsWith(fala, "<", true))
      {
         fala = fala.substr(0, fala.length() - 1);
         desfaz = false;
      }

      expr->mudaExpressaoFim(acao, desfaz);
   }
   return fala;
}

/* =================== Teste, no caso de nao haver audio =====================
int cntPalavra;
int fonemas[MAXCHAR];

// Simula o movimento dos labios no caso de nao haver audio
void TratamentoFala::simulaTratamentoFala()
{
   // Muda expressao antes de pronunciar o texto
   mudaExpressaoFacial(animIni, exprIni);

   // Desfaz o fonema anterior
   if (cntPalavra > 0)
      animVisemas[0]->interpolaInverso(fonemas[cntPalavra-1], INF_VIS, VEL_VIS);

   // Exibe o fonema atual
   animVisemas[1]->interpola(fonemas[cntPalavra], INF_VIS, VEL_VIS);

   // Quando as animacoes acima acabarem (acabam juntas, basta verificar uma)
   if (animVisemas[1]->encerrouAnimacao())
   {
      // Enquanto esta pronunciando a palavra, dispara o par de animacoes para
      // o proximo fonema
      if (cntPalavra < fala.length())
      {
         animVisemas[1]->iniciaAnimacao(0);
         animVisemas[0]->iniciaAnimacao(INF_VIS);
         cntPalavra++;
      }

      // Quando a palavra termina, apenas desfaz o ultimo fonema
      else if (cntPalavra == fala.length())
      {
         animVisemas[0]->iniciaAnimacao(INF_VIS);
         cntPalavra++;
      }
      else
      {
         // Ao terminar de processar o texto, exibe as expressoes que estao 
         // definidas no final da frase (se houver)
         mudaExpressaoFacial(animFim, exprFim);
      }
   }
}

// Simula aproximadamente os fonemas de uma frase, a partir dos seus caracteres
void TratamentoFala::simulaFonemas(String frase)
{
   for (int i = 0; i < frase.length(); i++)
   {
      if (frase.at(i) == 'A')
         fonemas[i+1] = VISEMA_A;

      else if (frase.at(i) == 'E')
         fonemas[i+1] = VISEMA_E;

      else if (frase.at(i) == 'I')
         fonemas[i+1] = VISEMA_I;

      else if (frase.at(i) == 'O')
         fonemas[i+1] = VISEMA_O;

      else if (frase.at(i) == 'U')
         fonemas[i+1] = VISEMA_U;

      else if (frase.at(i) == 'B' || frase.at(i) == 'M' 
            || frase.at(i) == 'P')
         fonemas[i+1] = VISEMA_BMP;

      else if (frase.at(i) == 'D' || frase.at(i) == 'S' 
            || frase.at(i) == 'T' || frase.at(i) == 'Z')
         fonemas[i+1] = VISEMA_DST;

      else if (frase.at(i) == 'F' || frase.at(i) == 'V')
         fonemas[i+1] = VISEMA_FV;

      else if (frase.at(i) == 'X' || frase.at(i) == 'J')
         fonemas[i+1] = VISEMA_JX;

      else if (frase.at(i) == 'K' || frase.at(i) == 'G'
            || frase.at(i) == 'Q' || frase.at(i) == 'C')
         fonemas[i+1] = VISEMA_KG;

      else if (frase.at(i) == 'R')
         fonemas[i+1] = VISEMA_R;

      else if (frase.at(i) == 'N' || frase.at(i) == 'L')
         fonemas[i+1] = VISEMA_LN;

      else
         fonemas[i+1] = 0;
   }
   // Dispara as animacoes
   animVisemas[0]->iniciaAnimacao(0);
   animVisemas[1]->iniciaAnimacao(0);

   // Zera o contador
   cntPalavra = 0;
}
*/
