#include "Visao.hpp"

// O objeto visao e um singleton, criado apenas uma vez
template<> Visao* Singleton<Visao>::ms_Singleton = 0;

IplImage* gray;
IplImage* small_img;

CvCapture* capture;

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
// Implementacao de thread no windows
//TODO

// Implementacao de thread no linux
#else
pthread_t visao;

// Cria uma thread para o tratamento da visao
void* threadVisao(void* param)
{
   IplImage* frame = 0;
   IplImage* copiaFrame = 0;

   if (capture)
   {
      while (1)
      {
         // Captura cada frame do video
         if (!cvGrabFrame(capture))
            break;
         frame = cvRetrieveFrame(capture);
         if (!frame)
            break;

         // Cria uma copia da imagem, para nao alterar diretamente a imagem 
         // capturada
         if (!copiaFrame)
            copiaFrame = cvCreateImage(cvSize(frame->width,frame->height),
                  IPL_DEPTH_8U, frame->nChannels);

         // Se a origem da imagem nao estiver no canto superior esquerdo, 
         // rotaciona em x para a imagem nao ficar de cabeca pra baixo
         if (frame->origin == IPL_ORIGIN_TL)
            cvCopy(frame, copiaFrame, 0);
         else
            cvFlip(frame, copiaFrame, 0);

         cvFlip(frame, copiaFrame, 1);

         // Deteccao das faces
         Visao::getSingleton().detecta(copiaFrame);

         if (cvWaitKey(10) >= 0)
            break;
      }
   }
}
#endif

// Construtor
Visao::Visao()
{
   chegou = false;
}

// Destrutor
Visao::~Visao()
{
   // Encerra a thread
   pthread_cancel(visao);
   
   // Libera as imagens
   if (capture)
   {
      cvReleaseCapture(&capture);
      cvReleaseImage(&gray);
      cvReleaseImage(&small_img);
   }

   // Destroi a janela
   cvDestroyWindow("Visao do Avatar");
}

// Inicializacao da camera
void Visao::inicializaVisao()
{
   // Carrega o arquivo da cascata de classificadores do tipo Haar features
   cascade = (CvHaarClassifierCascade*)
      cvLoad("../media/haar/haarcascade_frontalface_alt.xml", 0, 0, 0);

   if (!cascade)
      fprintf(stderr, "Erro ao carregar o arquivo de classificadores.\n");

   // Alocacao de memoria
   storage = cvCreateMemStorage(0);

   // Captura a imagem da camera 0
   capture = cvCaptureFromCAM(0);

   // Cria, redimensiona e posiciona uma janela para a imagem da camera
   cvNamedWindow("Visao do Avatar", 0);
   cvResizeWindow("Visao do Avatar", CAM_X, CAM_Y);
   cvMoveWindow("Visao do Avatar", (TELA_RES_X - CAM_X)/2,
         (TELA_RES_Y - CAM_Y)/2 - 300);

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
//TODO Implementacao de thread no windows
#else
   pthread_create(&visao, NULL, threadVisao, NULL);
#endif
}

// Metodo que realiza a deteccao das faces propriamente dita
void Visao::detecta(IplImage* img)
{
   // Cria a imagem a ser visualizada
   if (!gray)
      gray = cvCreateImage(cvSize(img->width,img->height), 8, 1);
   if (!small_img)
      small_img = cvCreateImage(cvSize(cvRound(img->width),
            cvRound(img->height)), 8, 1);

   cvCvtColor(img, gray, CV_BGR2GRAY);
   cvResize(gray, small_img, CV_INTER_LINEAR);
   cvEqualizeHist(small_img, small_img);
   cvClearMemStorage(storage);

   if (cascade)
   {
      // Detecta as faces armazenando-as do vetor faces
      CvSeq* faces = cvHaarDetectObjects(small_img, cascade, storage, 1.1, 2, 
            CV_HAAR_DO_CANNY_PRUNING, cvSize(30, 30));

      // Pega o numero de faces encontradas
      numFaces = faces->total;

      // Se nao achar nenhuma face
      if (numFaces == 0)
         chegou = false;

      // Se achar
      for (int i = 0; i < (faces ? numFaces : 0); i++)
      {
         // Pega o raio da face achada
         CvRect* r = (CvRect*)cvGetSeqElem(faces, i);
         int raio = cvRound((r->width + r->height) * 0.25);

         // Define um tamanho minimo para o raio
         if (raio > 70)
         {
            chegou = true;

            // Encontra o centro da face
            centroFace.x = cvRound((r->x + r->width * 0.5));
            centroFace.y = cvRound((r->y + r->height * 0.5));

            // Desenha um circulo de cor vermelha acordo com o centro da face e
            // o seu raio
            cvCircle(img, centroFace, raio, CV_RGB(255, 0, 0), 3, 8, 0);

            // Retorna apos a primeira face encontrada
            break;
         }
      }
   }
   // Mostra a imagem
   cvShowImage("Visao do Avatar", img);
}

// Metodos que retornam informacoes obtidas pela camera
bool Visao::usuarioChegou()
{
   return chegou;
}

int Visao::getNumFaces()
{
   return numFaces;
}

int Visao::getX()
{
   return centroFace.x;
}

int Visao::getY()
{
   return centroFace.y;
}


