#ifndef __VISAO_HPP__
#define __VISAO_HPP__

#include "cv.h"
#include "highgui.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include <time.h>
#include <ctype.h>

#include "Ogre.h"
#include "OgreSingleton.h"

#ifdef _EiC
#define WIN32
#endif

#define CAM_X 300
#define CAM_Y 226

#include "AplicacaoBase.hpp"

using namespace Ogre;

// Classe que trata da visao computacional do avatar, no caso, atraves do
// algoritmo de deteccao de faces do OpenCV
class Visao : public Singleton<Visao>
{
public:
   Visao();
   ~Visao();

   // Tratamento da camera
   void inicializaVisao();
   void detecta(IplImage* img);

   // Retorna informacoes obtidas pela camera
   bool usuarioChegou();
   int getNumFaces();
   int getX();
   int getY();

private:
   // Armazenamento de memoria
   CvMemStorage* storage;

   // Arquivo de classificadores
   CvHaarClassifierCascade* cascade;

   // Variavel que diz se o usuario esta em frente a camera
   bool chegou;

   // Controla o numero de faces achadas
   int numFaces;

   // Armazena a coordenada do centro da face encontrada
   CvPoint centroFace;
};

#endif
